# -*- coding: utf-8 -*-

FIAT_CURRENCY_AUD = u"AUD"
FIAT_CURRENCY_BRL = u"BRL"
FIAT_CURRENCY_CAD = u"CAD"
FIAT_CURRENCY_CHF = u"CHF"
FIAT_CURRENCY_CLP = u"CLP"
FIAT_CURRENCY_CNY = u"CNY"
FIAT_CURRENCY_CZK = u"CZK"
FIAT_CURRENCY_DKK = u"DKK"
FIAT_CURRENCY_EUR = u"EUR"
FIAT_CURRENCY_GBP = u"GBP"
FIAT_CURRENCY_HKD = u"HKD"
FIAT_CURRENCY_HUF = u"HUF"
FIAT_CURRENCY_IDR = u"IDR"
FIAT_CURRENCY_ILS = u"ILS"
FIAT_CURRENCY_INR = u"INR"
FIAT_CURRENCY_JPY = u"JPY"
FIAT_CURRENCY_KRW = u"KRW"
FIAT_CURRENCY_MXN = u"MXN"
FIAT_CURRENCY_MYR = u"MYR"
FIAT_CURRENCY_NOK = u"NOK"
FIAT_CURRENCY_NZD = u"NZD"
FIAT_CURRENCY_PHP = u"PHP"
FIAT_CURRENCY_PKR = u"PKR"
FIAT_CURRENCY_PLN = u"PLN"
FIAT_CURRENCY_RUB = u"RUB"
FIAT_CURRENCY_SEK = u"SEK"
FIAT_CURRENCY_SGD = u"SGD"
FIAT_CURRENCY_THB = u"THB"
FIAT_CURRENCY_TRY = u"TRY"
FIAT_CURRENCY_TWD = u"TWD"
FIAT_CURRENCY_ZAR = u"ZAR"

CRYPTO_CURRENCY_BTC = u"BTC"
CRYPTO_CURRENCY_ETH = u"ETH"
CRYPTO_CURRENCY_XRP = u"XRP"
CRYPTO_CURRENCY_LTC = u"LTC"
CRYPTO_CURRENCY_BCH = u"BCH"



import unittest
from coinmarketcap import CoinMarketCap


class TestCoinMarketCap(unittest.TestCase):
    """
    unit test case for CoinMarketCap lib
    """
    market = None

    def setUp(self):
        self.market = CoinMarketCap()
        super(TestCoinMarketCap, self).setUp()

    def test_coin_list(self):
        resp = self.market.coin_list()
        resp = self.market.coin_list(disable_cache=True)
        self.assertIsNotNone(resp)

    def test_stats(self):
        market = CoinMarketCap()
        resp = market.stats(convert="BTC")
        print(resp)
        self.assertIsNotNone(resp)
        self.assertIsNotNone(resp['data']['quotes']['BTC'])
        self.assertIsNone(resp['metadata']['error'])
        #

    def test_ticker(self):
        market = CoinMarketCap()
        resp = market.coin_detail(1, convert="CNY", start=1, limit=2, disable_cache=False)
        self.assertIsNotNone(resp)
        print(resp)

    def test_price(self):
        market = CoinMarketCap()
        resp = market.coin_price("eos")
        print(resp)

    def test_market_price(self):
        market = CoinMarketCap()
        resp = market.coin_market_price("eos")
        print(resp)

    def test_exchange_list(self):
        market = CoinMarketCap()
        resp = market.exchange_list()
        print(resp)

    def test_exchange_detail(self):
        market = CoinMarketCap()
        resp = market.exchange_detail()
        print(resp)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import tempfile
import json
import os
import requests
import requests_cache


class ApiClient(object):
    """
    coinmarketcap.com api wrapper with request cache feature
    """
    _session = None

    def __init__(self, request_timeout, enable_cache,
                 cache_filename, cache_expire_after):
        self.request_timeout = request_timeout
        self.enable_cache = enable_cache
        self.cache_filename = cache_filename
        self.cache_expire_after = cache_expire_after
        self.cache_file = os.path.join(tempfile.gettempdir(), self.cache_filename) if enable_cache else None

    @property
    def session(self):
        """
        wrapper for _session
        :return: 
        """
        if self._session is None:
            if self.enable_cache:
                requests_cache.install_cache(self.cache_file, backend='sqlite', expire_after=self.cache_expire_after)
            self._session = requests.Session()
            self._session.headers.update({'Content-Type': 'application/json'})
            self._session.headers.update(
                {'User-agent': 'coinmarketcap-api(https://github.com/shyuntech/coinmarketcap-api)'})
        return self._session

    def request(self, base_url, endpoint, params, disable_cache=False):
        print(base_url + endpoint)
        if disable_cache:
            with requests_cache.disabled():
                resp = self.session.get(base_url + endpoint, params=params, timeout=self.request_timeout)
        else:
            resp = self.session.get(base_url + endpoint, params=params, timeout=self.request_timeout)
        try:
            resp_json = json.loads(resp.text)
            if isinstance(resp_json, dict) and resp.status_code == 200:
                resp_json[u'cached'] = disable_cache
        except Exception as e:
            return e
        return resp_json

    def raw_request(self, base_url, endpoint, params, disable_cache=False):
        if disable_cache:
            disable_cache_session = self.session
            with requests_cache.disabled():
                resp = disable_cache_session.get(base_url + endpoint, params=params, timeout=self.request_timeout)
        else:
            resp = self.session.get(base_url + endpoint, params=params, timeout=self.request_timeout)

        return resp.text



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import re
from coinmarketcap import utils

from bs4 import BeautifulSoup as bs
from coinmarketcap.client import ApiClient


class CoinMarketCap(object):
    """
    
    market = CoinMarketCap()
    market.coin_list()
    """
    __CMC_API_URL = u'https://api.coinmarketcap.com/v2/'
    __CMC_HISTORY_API_URL = u'https://graphs2.coinmarketcap.com/'
    __CMC_BASE_URL = u'https://coinmarketcap.com/'
    __DEFAULT_REQUEST_TIMEOUT = 30
    __DEFAULT_ENABLE_CACHE = True
    __DEFAULT_CACHE_FILENAME = u'coinmarketcap.cache'
    __DEFAULT_CACHE_EXPIRE_AFTER = 30

    def __init__(self, enable_cache=__DEFAULT_ENABLE_CACHE, request_timeout=__DEFAULT_REQUEST_TIMEOUT,
                 cache_expire_after=__DEFAULT_CACHE_EXPIRE_AFTER, cache_filename=__DEFAULT_CACHE_FILENAME):
        self.client = ApiClient(request_timeout, enable_cache, cache_filename, cache_expire_after)

    def stats(self, **kwargs):
        """
        This endpoint displays the global data found at the top of coinmarketcap.com.

        Optional parameters:
        (string) convert - return pricing info in terms of another currency.
        Valid fiat currency values are: "AUD", "BRL", "CAD", "CHF", "CLP", "CNY", "CZK",
        "DKK", "EUR", "GBP", "HKD", "HUF", "IDR", "ILS", "INR", "JPY", "KRW", "MXN",
        "MYR", "NOK", "NZD", "PHP", "PKR", "PLN", "RUB", "SEK", "SGD", "THB", "TRY",
        "TWD", "ZAR"
        Valid cryptocurrency values are: "BTC", "ETH" "XRP", "LTC", and "BCH"
        """

        params = {}
        params.update(kwargs)
        response = self.client.request(self.__CMC_API_URL, u'global/', params)
        return response

    def coin_list(self, disable_cache=False):
        """
        This endpoint displays all active cryptocurrency listings in one call. Use the "id" field on 
        the Ticker endpoint to query more information on a specific cryptocurrency.
        
        GET /listings/
        
        :param disable_cache:  是否启用缓存
        :return: json
        """
        response = self.client.request(self.__CMC_API_URL, u'listings/', params=None, disable_cache=disable_cache)
        return response

    def coin_detail(self, currency, disable_cache=False, **kwargs):
        """
        This endpoint displays cryptocurrency ticker data in order of rank. The maximum 
        number of results per call is 100. Pagination is possible by using the start 
        and limit parameters.
        
        GET /ticker/
        
        Optional parameters:
            (int) start - return results starting from the specified number (default is 1)
            (int) limit - return a maximum of [limit] results (default is 100; max is 100)
            (string) sort - return results sorted by [sort] . Possible values are id, rank, volume_24h, and 
            percent_change_24h (default is rank). 
            Note: It is strongly recommended to use id to sort if paginating through all available results since id is the only sort option guaranteed to return in a consistent order.
            (string) structure - specify the structure for the main data field. Possible values are dictionary and array (default is dictionary).
            (string) convert - return pricing info in terms of another currency. 
            Valid fiat currency values are: "AUD", "BRL", "CAD", "CHF", "CLP", "CNY", "CZK", "DKK", "EUR", "GBP", "HKD", "HUF", "IDR", "ILS", "INR", "JPY", "KRW", "MXN", "MYR", "NOK", "NZD", "PHP", "PKR", "PLN", "RUB", "SEK", "SGD", "THB", "TRY", "TWD", "ZAR" 
            Valid cryptocurrency values are: "BTC", "ETH" "XRP", "LTC", and "BCH"

        :param currency: 货币
        :param disable_cache: 
        :return: 
        """
        params = {}
        params.update(kwargs)

        # see https://github.com/barnumbirr/coinmarketcap/pull/28
        if currency:
            currency = str(currency) + u'/'

        response = self.client.request(self.__CMC_API_URL, u'ticker/' + currency, params, disable_cache)
        return response

    def coin_price(self, currency, start=None, end=None, disable_cache=False):
        """
        获得货币的价格
        
        GET /currencies/<currency>/<start>/<end>
        GET /currencies/<currency>/
        
        Optional parameters:
            (int) start - return results starting from the specified timestamp
            (int) end - return results ending with the specified timestamp
            
        :param currency: 货币名称 必须是字符串
        :param disable_cache: 禁用缓存
        :param start: 
        :param end: 
        :return: 
        """
        if all((start, end)):
            endpoint = u'currencies/{}/{}/{}'.format(currency, start, end)
        else:
            endpoint = u'currencies/{}/'.format(currency)
        response = self.client.request(self.__CMC_HISTORY_API_URL, endpoint, None, disable_cache)

        return response

    def coin_market_price(self, currency, disable_cache=True):
        """
        https://coinmarketcap.com/currencies/<currency>/#markets
        
        :param currency: 
        :param disable_cache: 
        :return: 
        """
        endpoint = u'currencies/{}/#markets'.format(currency)
        response = self.client.raw_request(self.__CMC_BASE_URL, endpoint, None, disable_cache)

        soup = bs(response, u'html.parser')
        table_body = soup.find(u'table', {u'id': u'markets-table'}).find(u'tbody')
        rows = table_body.find_all(u'tr')
        for row in rows:
            tds = row.find_all(u'td')
            data = {
                u'exchange': tds[1][u'data-sort'],
                u'pair': tds[2][u'data-sort'],
                u'volume': utils._get_volume(tds[3]),
                u'price': utils._get_price(tds[4]),
                u'percentage': tds[5][u'data-sort']
            }
        return response

    def coin_ico_info(self):
        # TODO
        pass

    def exchange_list(self, disable_cache=False):
        """
        https://coinmarketcap.com/exchanges/volume/24-hour/all/ 
        This endpoint displays all active exchange listings in one call. 
        
        GET /exchanges/volume/24-hour/all/
        
        :param disable_cache:  是否启用缓存
        :return: json
        """
        endpoint = u'exchanges/volume/24-hour/all/'
        response = self.client.raw_request(self.__CMC_BASE_URL, endpoint, None, disable_cache)
        soup = bs(response, u'html.parser')

        rows = soup.find_all(u'h3', {u'class': u'padding-top--lv6 margin-bottom--lv2'})
        items = []
        for idx, row in enumerate(rows, start=1):
            item = {
                u'display_name': row.find(u'a', {u'class': u'link-secondary'}).get_text(),
                u'name': utils._get_name_form_url(row.find(u'a', {u'class': u'link-secondary'})[u'href']),
                u'url': row.find(u'a', {u'class': u'link-secondary'})[u'href'],
                u'logo': row.find(u'img', {u'class': u'logo-32x32'})[u'src'],
                u'rank': idx,
            }
            items.append(item)
        resp = {
            u'data': items,
            u"metadata": {
                u"num_exchanges": len(items),
                u"error": None
            }
        }
        return resp

    def exchange_detail(self):
        # TODO
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import re


def _get_name_form_url(url):
    return re.match(r'/exchanges/(.*)/', url).group(1)


def _get_price(td):
    try:
        price_span = td.find(u"span", {u"class": u"price"})
        price = {
            u'price_usd': price_span[u"data-usd"],
            u'price_btc': price_span[u"data-btc"],
            u'price_native': price_span[u'data-native'],
        }
        return price
    except Exception as error:
        return None


def _get_volume(td):
    try:
        price_span = td.find(u"span", {u'class': u'volume'})
        volume = {
            u'volume_usd': price_span[u'data-usd'],
            u'volume_btc': price_span[u'data-btc'],
            u'volume_native': price_span[u'data-native'],
        }
        return volume
    except Exception as error:
        return None



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

version = '0.0.1'

LONG_DESCRIPTION = """
=====================================
coinmarketcap-api (coinmarketcap-api)
=====================================
Python实现的 coinmarketcap.com APIs封装，同时提供了更多私有API实现
涵盖整体市场数据、虚拟货币数据、交易所数据等

Python wrapper for coinmarketcap.com public API and private API
offer global market data,coin data,exchanges data and more
Documentation
=============
https://github.com/shyuntech/coinmarketcap-api
"""

setup(
    name='coinmarketcap-api',
    version=version,
    description="Python wrapper for coinmarketcap.com public API and private API"
                "offer global market data,coin data,exchanges data and more",
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    classifiers=[
        'License :: OSI Approved :: Apache Software License',
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    keywords=['cryptocurrency', 'API', 'coinmarketcap'],
    author='hu chi',
    author_email='hu.chi@qq.com',
    url='http://github.com/shuyun/coinmarketcap-api',
    license='Apache v2.0 License',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
)
